/*BEGIN COPYRIGHT */
/**************************************************************************
*
*   Licensed Materials - Property of IBM
*   5697-FM3 5655-A96
*   (C) Copyright IBM Corp. 1998, 2000
*   US Government Users Restricted Rights - Use, duplication or
*   disclosure restricted by GSA ADP Schedule Contract
*   with IBM Corp.
*
**************************************************************************/
/*END COPYRIGHT */

/* BEGIN DISCLAIMER */
/**********************************************************************
*  DISCLAIMER
*  ----------
*  This material contains programming source code for your consideration.
*  These examples have not been thoroughly tested under all conditions.
*  IBM, therefore, cannot guarantee or imply reliability, serviceability,
*  or function of these programs.
*  ALL PROGRAMS CONTAINED HEREIN ARE PROVIDED TO YOU "AS IS", WITHOUT ANY
*  WARRANTIES (EXPRESS OR IMPLIED) OR SUPPORT WHATSOEVER, INCLUDING BUT
*  NOT LIMITED TO ANY IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS
*  FOR A PARTICULAR PURPOSE.
**********************************************************************/
/* END DISCLAIMER */

/**********************************************************************/
/* MQ Series Workflow API Examples - Check in / check out workitems   */
/*                                                                    */
/* This sample demonstrates how workitems can be checked out of the   */
/* normal MQS workflow environment so that the data contained in the  */
/* input container can be handled externally.                         */
/**********************************************************************/

#include <fmctjinc.hxx>                 // compiler-dependent #includes
#include <fmcjprun.hxx>                 // MQWF C++ API interface
#include <ctype.h>                      // character classification


int main (int argc, char ** argv)
{
  FmcjExecutionService                  service;

  unsigned long                         i,j,k           = 0;

  string                                choice          ="";
  int                                   usrchoice       = 0;

/*--------------------------------------------------------------------*/
/* Check command line parameters, logon to system                     */
/*--------------------------------------------------------------------*/
  if (argc!=3)
  {
    cout << "\nUsage: " << argv[0] << " <UserID> <Password>" << endl;
    return -1;
  }
  cout << endl;
  APIRET rc = FmcjGlobal::Connect();
  FmcjResult * result = FmcjResult::ObjectOfCurrentThread();
  rc = service.Logon( argv[1], argv[2], FmcjService::Default );
  if (rc != FMC_OK)
  {
    cout << "Logon failed with RC " << rc << " - " << result->MessageText()
         << endl;
    // Deinit API environment
    FmcjGlobal::Disconnect();
    delete(result);
    return rc;
  }
  cout << "Logged on with UserID '" << service.UserID()
       << "' on system '" << service.SystemName()
       << "' at system group '" << service.SystemGroupName()
       << "'." << endl;

/*-------------------------------------------------------------------*/
/* Display a list and prompt user for action                         */
/*-------------------------------------------------------------------*/
  vector<FmcjWorkitem>           wList;
  unsigned long                  threshold       = 20;
  string                         WIListSort      ="NAME ASC";
  string                         WIListFilter    ="OWNER=CURRENT_USER";
  FmcjReadWriteContainer         checkInOutput;
  FmcjProgramData                checkOutData;

  do                                    // Beginning of main tool loop
  {
    cout << endl;

    // Clear workitem vector
    wList.erase(wList.begin(),wList.end());

    // Set filter criteria: only those workitems which the logged-on
    // user owns will be displayed.

    rc = service.QueryWorkitems( &WIListFilter,
                                 &WIListSort,
                                 &threshold,
                                 wList );

    if (rc != FMC_OK)
    {
      cout << "Querying workitems failed with RC " << rc << " - "
           << result->MessageText() << endl;
      service.Logoff();
      // Deinit API environment
      FmcjGlobal::Disconnect();
      delete(result);
      return -1;
    }

    // Display as many workitems as specified with threshold

    cout << endl;
    cout << "#   State          Workitem name" << endl;
    cout << "--------------------------------" << endl;

    if (wList.size() > 0 )
    {
      for( i=0; i<wList.size(); i++ )
      {
        cout << i;
        if (i<10) cout << " ";
        cout << "  ";

        switch (wList[i].State())
        {
          case FmcjItem::undefined: cout << "undefined    "; break;
          case FmcjItem::ready: cout << "ready        "; break;
          case FmcjItem::finished: cout << "finished     "; break;
          case FmcjItem::suspended: cout << "suspended    "; break;
          case FmcjItem::checkedOut: cout << "checkedOut   "; break;
          case FmcjItem::executed: cout << "executed     "; break;
          case FmcjItem::forceFinished: cout << "forceFinished"; break;
          case FmcjItem::terminating: cout << "terminating  "; break;
          case FmcjItem::running: cout << "running      "; break;
          case FmcjItem::terminated: cout << "terminated   "; break;
          case FmcjItem::disabled: cout << "disabled     "; break;
          case FmcjItem::inError: cout << "inError      "; break;
          case FmcjItem::planning: cout << "planning     "; break;
          case FmcjItem::deleted: cout << "deleted      "; break;
          case FmcjItem::suspending: cout << "suspending   "; break;
        }
        cout << "  " << wList[i].Name() << endl;
      }
    }
    else cout << "   <no workitems for user '" << argv[1] << "' found>" << endl;

    // Show actions which user can perform and query user

    cout << endl
         << "Commands: [o] Check out workitem            [v] View workitem info\n"
         << "          [i] Check in workitem             [q] Quit program\n"
         << "[o/i/v/q]? ";
    cout.flush();

    cin >> choice;

    usrchoice=0;

    if (strncmp(choice.c_str(),"o",1)==0) usrchoice=1;
    if (strncmp(choice.c_str(),"i",1)==0) usrchoice=2;
    if (strncmp(choice.c_str(),"v",1)==0) usrchoice=3;
    if (strncmp(choice.c_str(),"q",1)==0) usrchoice=255;


    if (wList.size()>0)
    switch (usrchoice)
    {
      case 1:
/*-------------------------------------------------------------------*/
/* Check out a workitem.                                             */
/* In order to do that, the associated process instance must be in   */
/* the RUNNING state, the workitem must be in the READY state and    */
/* must be implemented by a program. In this example, the            */
/* implementation data for the Windows NT platform are queried.      */
/*-------------------------------------------------------------------*/
        do
        {
          cout << "Please enter workitem number: ";
          cout.flush();
          cin >> choice;
          j=0;
          for (i=0; i<strlen(choice.c_str()); i++)
          { if (isdigit(choice.c_str()[i])) j++; }
        } while (i!=j);

        i = atoi(choice.c_str());
        cout << endl;

        if (i>(wList.size()-1) && wList.size()>0 )
        {
          cout << "Please use range 0-" << wList.size()-1 << "." << endl;
        }
        else
        {
          rc = wList[i].CheckOut( FmcjWorkitem::SpecifiedDefinitions,
                                  FmcjImplementationData::WindowsNT,
                                  checkOutData );

          if ( rc!=FMC_OK )
          {
            cout << "Checking out workitem failed with RC " << rc << " - "
                 << result->MessageText() << endl << endl;
          }
          else
          {
            // Now it's possible to get the input and output container
            // from the FmcjProgramData object
            FmcjReadOnlyContainer               * checkOutinctnr;
            FmcjReadWriteContainer              * checkOutoutctnr;
            vector<FmcjImplementationData>        impData;

            checkOutinctnr  = checkOutData.InContainer();
            checkOutoutctnr = checkOutData.OutContainer();

            // Clear FmcjImplementationData vector

            impData.erase(impData.begin(),impData.end());

            checkOutData.Implementations(impData);

            cout << "Workitem has been checked out.\n"
                 << "Input Container name : "
                 << checkOutinctnr->Type() << "\n"
                 << "Output Container name: "
                 << checkOutoutctnr->Type() << "\n"
                 << "Execution user       : "
                 << checkOutData.ExecutionUser() << "\n\n"
                 << endl;
            cout << "Windows NT Implementation Data\n"
                 << "------------------------------\n";

            FmcjDllOptions * dllOptions;
            FmcjExeOptions * exeOptions;
            FmcjExternalOptions * externalOptions;

            switch (impData[0].Kind())
            {
              case FmcjImplementationData::DLL:

                // Fetch DLL implementation data

                dllOptions = impData[0].DllOptions();

                cout << "Path and file name: "
                     << dllOptions->PathAndFileName() << "\n"
                     << "Entry point name  : "
                     << dllOptions->EntryPointName() << endl;

                break;

              case FmcjImplementationData::EXE:

                // Fetch EXE implementation data

                exeOptions = impData[0].ExeOptions();

                cout << "Path and file name: "
                     << exeOptions->PathAndFileName() << "\n"
                     << "Working directory : "
                     << exeOptions->WorkingDirectoryName() << endl;

                break;

              case FmcjImplementationData::External:

                // Fetch external implementation data

                externalOptions = impData[0].ExternalOptions();

                cout << "Executable name: "
                     << externalOptions->ExecutableName() << "\n"
                     << "Invocation type: "
                     << externalOptions->InvocationType() << endl;

                break;
            }
          }
        }

        break;


      case 2:
/*-------------------------------------------------------------------*/
/* Check in a workitem                                               */
/*-------------------------------------------------------------------*/
        do
        {
          cout << "Please enter workitem number: ";
          cout.flush();
          cin >> choice;
          j=0;
          for (i=0; i<strlen(choice.c_str()); i++)
          { if (isdigit(choice.c_str()[i])) j++; }
        } while (i!=j);

        i = atoi(choice.c_str());
        cout << endl;

        if (i>(wList.size()-1) && wList.size()>0 )
        {
          cout << "Please use range 0-" << wList.size()-1 << "." << endl;
        }
        else
        {

          do
          {
            cout << "Enter desired RC of workitem: ";
            cout.flush();
            cin >> choice;
            j=0;
            for (k=0; k<strlen(choice.c_str()); k++)
            { if (isdigit(choice.c_str()[i])) j++; }
          } while (k!=j);

          j = atoi(choice.c_str());
          cout << endl;

          // Get ReadWriteContainer from workitem
          rc = wList[i].OutContainer(checkInOutput);
          if ( rc!=FMC_OK )
          {
            cout << "Getting Output Container from workitem failed with RC "
                 << rc << " - " << result->MessageText() << endl << endl;
          }

          // Check in workitem
          rc = wList[i].CheckIn( &checkInOutput, j );
          if ( rc!=FMC_OK )
          {
            cout << "Checking in workitem failed with RC " << rc << " - "
                 << result->MessageText() << endl << endl;
          }

        }

        break;


      case 3:
/*-------------------------------------------------------------------*/
/* Show some workitem info                                           */
/*-------------------------------------------------------------------*/
        do
        {
          cout << "Please enter workitem number: ";
          cout.flush();
          cin >> choice;
          j=0;
          for (i=0; i<strlen(choice.c_str()); i++)
          { if (isdigit(choice.c_str()[i])) j++; }
        } while (i!=j);

        i = atoi(choice.c_str());
        cout << endl;

        if (i>(wList.size()-1) && wList.size()>0 )
        {
          cout << "Please use range 0-" << wList.size()-1 << "." << endl;
        }
        else
        {
          cout << "Workitem name         : "
               << wList[i].Name() << "\n"
               << "Process instance name : "
               << wList[i].ProcessInstanceName() << "\n";
          cout << "Creation time         : "
               << string(wList[i].CreationTime()) << "\n"
               << "Program implementation: "
               << wList[i].Implementation() << "\n";

          if (!wList[i].ErrorReasonIsNull())
          {
            FmcjError error = wList[i].ErrorReason();
            cout << "\nError RC  : " << error.Rc() << "\n";
            cout << "Error text: " << error.MessageText();
          }
        }

        break;

    }                                   // end switch
    else cout << endl << "No workitems available!" << endl;

  } while ( (usrchoice!=255) );


/*-------------------------------------------------------------------*/
/* Logoff and deinit the API environment.                            */
/*-------------------------------------------------------------------*/

  cout << "\nLogging off." << endl;
  rc = service.Logoff();
  cout << "Deinitializing API environment." << endl;
  rc = FmcjGlobal::Disconnect();
  delete(result);
  return 0;
}
