/*BEGIN COPYRIGHT */
/**************************************************************************
*
*   Licensed Materials - Property of IBM
*   5697-FM3 5655-A96
*   (C) Copyright IBM Corp. 1998, 2000
*   US Government Users Restricted Rights - Use, duplication or
*   disclosure restricted by GSA ADP Schedule Contract
*   with IBM Corp.
*
**************************************************************************/
/*END COPYRIGHT */

/* BEGIN DISCLAIMER */
/**********************************************************************
*  DISCLAIMER
*  ----------
*  This material contains programming source code for your consideration.
*  These examples have not been thoroughly tested under all conditions.
*  IBM, therefore, cannot guarantee or imply reliability, serviceability,
*  or function of these programs.
*  ALL PROGRAMS CONTAINED HEREIN ARE PROVIDED TO YOU "AS IS", WITHOUT ANY
*  WARRANTIES (EXPRESS OR IMPLIED) OR SUPPORT WHATSOEVER, INCLUDING BUT
*  NOT LIMITED TO ANY IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS
*  FOR A PARTICULAR PURPOSE.
**********************************************************************/
/* END DISCLAIMER */

/**********************************************************************/
/* MQS Workflow API Examples - Handling workitems and notifications   */
/*                                                                    */
/* This program shows how to handle workitems and notifications.      */
/**********************************************************************/

#include <fmctjinc.hxx>                 // compiler-dependent #includes
#include <fmcjprun.hxx>                 // MQWF C++ API interface
#include <ctype.h>                      // character classification


int main (int argc, char ** argv)
{

  unsigned long                         i               = 0;
  unsigned long                         j               = 0;

  FmcjExecutionService                  service;

  string                                choice          ="";
  string                                ctrChoice       ="Y";
  int                                   usrchoice       = 0;
  vector<string>                        info;

  FmcjError                             error;


/*--------------------------------------------------------------------*/
/* Check command line parameters, logon to system                     */
/*--------------------------------------------------------------------*/

  if (argc!=3)
  {
    cout << "\nUsage: " << argv[0] << " <UserID> <Password>" << endl;
    return -1;
  }
  cout << endl;
  APIRET rc = FmcjGlobal::Connect();
  FmcjResult * result = FmcjResult::ObjectOfCurrentThread();
  rc = service.Logon( argv[1], argv[2], FmcjService::Default );
  if (rc != FMC_OK)
  {
    cout << "Logon failed with RC " << rc << " - " << result->MessageText()
         << endl;
    // Deinit API environment
    FmcjGlobal::Disconnect();
    delete(result);
    return -1;
  }
  cout << "Logged on with UserID '" << service.UserID()
       << "' on system '" << service.SystemName()
       << "' at system group '" << service.SystemGroupName()
       << "'." << endl;

/*-------------------------------------------------------------------*/
/* Start a PEA to permit starting workitems                          */
/*-------------------------------------------------------------------*/

  cout << "\nStarting PEA for user '" << service.UserID() << "'... " << endl;
  rc = service.PEAStartUp();
  if (rc!=FMC_OK && rc!=FMC_ERROR_ALREADY_STARTED )
  {
    cout << "Starting PEA for user '" << service.UserID() << "' failed with RC "
         << rc << " - " << result->MessageText() << endl;
    // Deinit API environment
    FmcjGlobal::Disconnect();
    delete(result);
    return -1;
  }
  bool PEAstarted=false;
  if (rc==FMC_ERROR_ALREADY_STARTED)
  {
    cout << "PEA is already started." << endl;
    PEAstarted=false;
  }
  else PEAstarted=true;


/*-------------------------------------------------------------------*/
/* Based on user's choice, display available workitems, process      */
/* instance notifications or activity instance notifications.        */
/*-------------------------------------------------------------------*/

  do                                    // Beginning of main loop
  {
    cout << endl;
    cout << "Do you want to view the available [w]orkitems?\n"
         << "                                  [p]rocess instance notifications?\n"
         << "                                  [a]ctivity instance notifications?\n"
         << "[w/p/a]? "; cout.flush();

    cin >> choice;

    FmcjReadOnlyContainer                       input;
    FmcjReadWriteContainer                      output;
    FmcjWorkitem                                workitem;
    vector<FmcjWorkitem>                        vwList;
    string                                      listName        ="";
    string                                      workitemName    ="";
    string                                      WIListSort      ="";
    string                                      WIListFilter    ="";

    if (strncmp(choice.c_str(),"w",1)==0)
    {
      // Clear vector
      vwList.erase(vwList.begin(),vwList.end());
      // Set sorting criterium
      WIListSort   = "NAME ASC";
      // Set filter criteria: only those workitems which the logged-on
      // user owns will be displayed.
      WIListFilter = "OWNER=CURRENT_USER";
      rc = service.QueryWorkitems( &WIListFilter,
                                   &WIListSort,
                                   FmcjNoThreshold,
                                   vwList );
      if (rc != FMC_OK)
      {
        cout << "Querying workitems failed with RC " << rc << " - "
             << result->MessageText() << endl;
        service.Logoff();
        // Deinit API environment
        FmcjGlobal::Disconnect();
        delete(result);
        return -1;
      }
      cout << "\n                   Support\n"
           << "#   State          Tool?    Workitem name\n"
           << "-----------------------------------------" << endl;
      if (vwList.size() > 0 )
      {
        for( i=0; i<vwList.size(); i++ )
        {
          cout << i;
          if (i<10) cout << " ";
          cout << "  ";
          switch (vwList[i].State())
          {
            case FmcjItem::undefined: cout << "undefined    "; break;
            case FmcjItem::ready: cout << "ready        "; break;
            case FmcjItem::finished: cout << "finished     "; break;
            case FmcjItem::suspended: cout << "suspended    "; break;
            case FmcjItem::checkedOut: cout << "checkedOut   "; break;
            case FmcjItem::executed: cout << "executed     "; break;
            case FmcjItem::forceFinished: cout << "forceFinished"; break;
            case FmcjItem::terminating: cout << "terminating  "; break;
            case FmcjItem::running: cout << "running      "; break;
            case FmcjItem::terminated: cout << "terminated   "; break;
            case FmcjItem::disabled: cout << "disabled     "; break;
            case FmcjItem::inError: cout << "inError      "; break;
            case FmcjItem::planning: cout << "planning     "; break;
            case FmcjItem::deleted: cout << "deleted      "; break;
            case FmcjItem::suspending: cout << "suspending   "; break;
            case FmcjItem::expired: cout << "expired      "; break;
            default: cout << vwList[i].State();
          }
          cout << "  ";
          if (vwList[i].SupportToolsIsNull()) cout << "no ";
          else                           cout << "yes";
          cout << "    " << vwList[i].Name() << endl;
        }
      }
      else cout << "   <no workitems for user '" << service.UserID() << "' found>" << endl;
      // Show actions which user can perform
      cout << endl
           << "Commands: [v] View detailed workitem info       [i] Finish workitem\n"
           << "          [s] Start workitem                    [c] Force finish workitem\n"
           << "          [r] Restart workitem                  [o] Start support tool\n"
           << "          [f] Force restart workitem            [n] Reload a list\n"
           << "          [t] Terminate workitem                [q] Quit program\n"
           << "          [d] Delete workitem                                   \n"
           << "[v/s/r/f/t/i/c/o/n/q/d]? ";
      cout.flush();
    } /* end if: display workitems */

    FmcjProcessInstanceNotification             piNot;
    vector<FmcjProcessInstanceNotification>     piNotList;
    string                                      piNotSort       ="";
    string                                      piNotFilter     ="";
    string                                      piNotName       ="";

    if (strncmp(choice.c_str(),"p",1)==0)
    {
      // Clear vector
      piNotList.erase(piNotList.begin(),piNotList.end());
      // Set sorting criterium
      piNotSort   = "NAME ASC";
      rc = service.QueryProcessInstanceNotifications( FmcjNoFilter,
                                                      &piNotSort,
                                                      FmcjNoThreshold,
                                                      piNotList );
      if (rc != FMC_OK)
      {
        cout << "Querying process instance notifications failed with RC "
             << rc << " - " << result->MessageText() << endl;
        service.Logoff();
        // Deinit API environment
        FmcjGlobal::Disconnect();
        delete(result);
        return -1;
      }
      cout << endl;
      cout << "#   Creation Time     Notification for" << endl;
      cout << "--------------------------------------" << endl;
      if (piNotList.size() > 0 )
      {
        for( i=0; i<piNotList.size(); i++ )
        {
          cout << i;
          if (i<10) cout << " ";
          cout << "  " << string(piNotList[i].CreationTime())
               << "  " << piNotList[i].Name() << endl;
        }
      }
      else cout << "   <no process instance notifications for user '"
                << service.UserID() << "' found>" << endl;
      // Show actions which user can perform
      cout << endl
           << "Commands: [p] View process instance              [n] Reload a list\n"
           << "              notification info                  [q] Quit program\n"
           << "[p/n/q]? ";
      cout.flush();
    } /* end if: display process instance notifications */


    FmcjActivityInstanceNotification            aiNot;
    vector<FmcjActivityInstanceNotification>    aiNotList;
    string                                      aiNotSort       ="";
    string                                      aiNotFilter     ="";
    string                                      aiNotName       ="";

    if (strncmp(choice.c_str(),"a",1)==0)
    {
      // Clear vector
      aiNotList.erase(aiNotList.begin(),aiNotList.end());
      // Set sorting criterium
      aiNotSort   = "NAME ASC";
      rc = service.QueryActivityInstanceNotifications( FmcjNoFilter,
                                                       &aiNotSort,
                                                       FmcjNoThreshold,
                                                       aiNotList );
      if (rc != FMC_OK)
      {
        cout << "Querying activity instance notifications failed with RC "
             << rc << " - " << result->MessageText() << endl;
        service.Logoff();
        // Deinit API environment
        FmcjGlobal::Disconnect();
        delete(result);
        return -1;
      }
      cout << endl;
      cout << "#   Creation Time     Notification for" << endl;
      cout << "--------------------------------------" << endl;

      if ( aiNotList.size()>0 )
      {
        for( i=0; i<aiNotList.size(); i++ )
        {
          cout << i;
          if (i<10) cout << " ";
          cout << "  " << string(aiNotList[i].CreationTime())
               << "  " << aiNotList[i].Name() << endl;
        }
      }
      else cout << "   <no activity instance notifications for user '"
                << service.UserID() << "' found>" << endl;
      // Show actions which user can perform
      cout << endl
           << "Commands: [a] View activity instance             [n] Reload a list\n"
           << "              notification info                  [q] Quit program\n"
           << "[a/n/q]? ";
      cout.flush();
    } /* end if: display activity instance notifications */

    // Query user for action
    cin >> choice;
    usrchoice=0;
    if (strncmp(choice.c_str(),"v",1)==0) usrchoice=1;
    if (strncmp(choice.c_str(),"s",1)==0) usrchoice=2;
    if (strncmp(choice.c_str(),"r",1)==0) usrchoice=3;
    if (strncmp(choice.c_str(),"f",1)==0) usrchoice=4;
    if (strncmp(choice.c_str(),"t",1)==0) usrchoice=5;
    if (strncmp(choice.c_str(),"i",1)==0) usrchoice=6;
    if (strncmp(choice.c_str(),"c",1)==0) usrchoice=7;
    if (strncmp(choice.c_str(),"p",1)==0) usrchoice=8;
    if (strncmp(choice.c_str(),"a",1)==0) usrchoice=9;
    if (strncmp(choice.c_str(),"n",1)==0) usrchoice=10;
    if (strncmp(choice.c_str(),"o",1)==0) usrchoice=11;
    if (strncmp(choice.c_str(),"d",1)==0) usrchoice=12;
    if (strncmp(choice.c_str(),"q",1)==0) usrchoice=255;


    switch (usrchoice)
    {
      case 1:
/*-------------------------------------------------------------------*/
/* Show some workitem info                                           */
/*-------------------------------------------------------------------*/

        do
        {
          cout << "Please enter workitem number: ";
          cout.flush();
          cin >> choice;
          j=0;
          for (i=0; i<strlen(choice.c_str()); i++)
          { if (isdigit(choice.c_str()[i])) j++; }
        } while (i!=j);

        i = atoi(choice.c_str());
        cout << endl;

        if (i>(vwList.size()-1) && vwList.size()>0 )
        {
          cout << "Please use range 0-" << vwList.size()-1 << "." << endl;
        }
        else
        {
          // Clear string vector
          info.erase(info.begin(),info.end());
          workitem = vwList[i];
          workitem.Refresh();           // Fetch secondary properties

          workitem.SupportTools(info);

          cout << "Workitem name         : "
               << workitem.Name() << "\n"
               << "Process instance name : "
               << workitem.ProcessInstanceName() << "\n";
          cout << "Creation time         : "
               << string(workitem.CreationTime()) << "\n"
               << "Program implementation: "
               << workitem.Implementation() << "\n"
               << "Support tools         : ";

          for (j=0; j<info.size(); j++)
          {
             if   (j!=(info.size()-1)) cout << info[j] << ", ";
             else                      cout << info[j] << endl;
          }

          cout << "\nFirst notification    : "
               << string(workitem.FirstNotificationTime()) << "\n"
               << "Second notification   : "
               << string(workitem.SecondNotificationTime()) << endl;

          // Clear string vector
          info.erase(info.begin(),info.end());
          workitem.Staff(info);

          cout << "Workitem Staff        : ";

          for (j=0; j<info.size(); j++)
          {
             if   (j!=(info.size()-1)) cout << info[j] << ", ";
             else                      cout << info[j] << endl;
          }

          if (!workitem.ErrorReasonIsNull())
          {
            error = workitem.ErrorReason();
            cout << "\nError RC  : " << error.Rc() << "\n";
            cout << "Error text: " << error.MessageText();
          }
        }
        break;


      case 2:
/*-------------------------------------------------------------------*/
/* Start a workitem                                                  */
/*-------------------------------------------------------------------*/

        do
        {
          cout << "Please enter workitem number: ";
          cout.flush();
          cin >> choice;
          j=0;
          for (i=0; i<strlen(choice.c_str()); i++)
          { if (isdigit(choice.c_str()[i])) j++; }
        } while (i!=j);

        i = atoi(choice.c_str());
        cout << endl;

        if (i>(vwList.size()-1) && vwList.size()>0)
        {
          cout << "Please use range 0-" << vwList.size()-1 << "." << endl;
        }
        else
        {
          workitem = vwList[i];

          rc = workitem.Start();

          if (rc != FMC_OK)
          {
            cout << "Starting workitem failed with RC " << rc << " - "
                 << result->MessageText() << endl;
          }
        }
        break;


      case 3:
/*-------------------------------------------------------------------*/
/* Restart workitem                                                  */
/*-------------------------------------------------------------------*/

        do
        {
          cout << "Please enter workitem number: ";
          cout.flush();
          cin >> choice;
          j=0;
          for (i=0; i<strlen(choice.c_str()); i++)
          { if (isdigit(choice.c_str()[i])) j++; }
        } while (i!=j);

        i = atoi(choice.c_str());
        cout << endl;

        if (i>(vwList.size()-1) && vwList.size()>0)
        {
          cout << "Please use range 0-" << vwList.size()-1 << "." << endl;
        }
        else
        {
          workitem = vwList[i];

          rc = workitem.Restart();

          if (rc != FMC_OK)
          {
            cout << "Restarting workitem failed with RC " << rc << " - "
                 << result->MessageText() << endl;
          }
        }
        break;


      case 4:
/*-------------------------------------------------------------------*/
/* Force restart workitem                                            */
/*-------------------------------------------------------------------*/

        do
        {
          cout << "Please enter workitem number: ";
          cout.flush();
          cin >> choice;
          j=0;
          for (i=0; i<strlen(choice.c_str()); i++)
          { if (isdigit(choice.c_str()[i])) j++; }
        } while (i!=j);

        i = atoi(choice.c_str());
        cout << endl;

        if (i>(vwList.size()-1) && vwList.size()>0)
        {
          cout << "Please use range 0-" << vwList.size()-1 << "." << endl;
        }
        else
        {
          workitem = vwList[i];

          cout << endl
               << "Do you want to provide an input container? [Y/N] ";
          cout.flush();
          cin >> ctrChoice;

          if (strncmp(ctrChoice.c_str(),"Y",1)==0)
          /* ForceRestart with input container.                     */
          /* Obtain input container.                                */
          /* Insert code here to modify ("repair") container values */
          {
            rc = workitem.InContainer( input );
            rc = workitem.ForceRestart( input );
          }
          else
          {
            rc = workitem.ForceRestart();
          }

          if (rc != FMC_OK)
          {
            cout << "Forced restarting workitem failed with RC " << rc << " - "
                 << result->MessageText() << endl;
          }
        }
        break;


      case 5:
/*-------------------------------------------------------------------*/
/* Terminate workitem                                                */
/*-------------------------------------------------------------------*/

        do
        {
          cout << "Please enter workitem number: ";
          cout.flush();
          cin >> choice;
          j=0;
          for (i=0; i<strlen(choice.c_str()); i++)
          { if (isdigit(choice.c_str()[i])) j++; }
        } while (i!=j);

        i = atoi(choice.c_str());
        cout << endl;

        if (i>(vwList.size()-1) && vwList.size()>0)
        {
          cout << "Please use range 0-" << vwList.size()-1 << "." << endl;
        }
        else
        {
          workitem = vwList[i];

          rc = workitem.Terminate();

          if (rc != FMC_OK)
          {
            cout << "Terminating workitem failed with RC " << rc << " - "
                 << result->MessageText() << endl;
          }
        }
        break;


      case 6:
/*-------------------------------------------------------------------*/
/* Finish workitem                                                   */
/*-------------------------------------------------------------------*/

        do
        {
          cout << "Please enter workitem number: ";
          cout.flush();
          cin >> choice;
          j=0;
          for (i=0; i<strlen(choice.c_str()); i++)
          { if (isdigit(choice.c_str()[i])) j++; }
        } while (i!=j);

        i = atoi(choice.c_str());
        cout << endl;

        if (i>(vwList.size()-1) && vwList.size()>0)
        {
          cout << "Please use range 0-" << vwList.size()-1 << "." << endl;
        }
        else
        {
          workitem = vwList[i];

          rc = workitem.Finish();

          if (rc != FMC_OK)
          {
            cout << "Finishing workitem failed with RC " << rc << " - "
                 << result->MessageText() << endl;
          }
        }
        break;


      case 7:
/*-------------------------------------------------------------------*/
/* Force finish workitem                                             */
/*-------------------------------------------------------------------*/

        do
        {
          cout << "Please enter workitem number: ";
          cout.flush();
          cin >> choice;
          j=0;
          for (i=0; i<strlen(choice.c_str()); i++)
          { if (isdigit(choice.c_str()[i])) j++; }
        } while (i!=j);

        i = atoi(choice.c_str());
        cout << endl;

        if (i>(vwList.size()-1) && vwList.size()>0)
        {
          cout << "Please use range 0-" << vwList.size()-1 << "." << endl;
        }
        else
        {
          workitem = vwList[i];

          cout << endl
               << "Do you want to provide an output container? [Y/N] ";
          cout.flush();
          cin >> ctrChoice;

          if (strncmp(ctrChoice.c_str(),"Y",1)==0)
          /* ForceFinish with output container.                     */
          /* Obtain output container.                               */
          /* Insert code here to modify ("repair") container values */
          {
            rc = workitem.OutContainer( output );
            rc = output.SetValue("_RC", (long)FMC_OK );
            rc = workitem.ForceFinish( output );
          }
          else
          {
            rc = workitem.ForceFinish();
          }

          if (rc != FMC_OK)
          {
            cout << "Forced finishing workitem failed with RC " << rc << " - "
                 << result->MessageText() << endl;
          }
        }
        break;


      case 8:
/*-------------------------------------------------------------------*/
/* Show process instance notification info                           */
/*-------------------------------------------------------------------*/

        do
        {
          cout << "Please enter notification number: ";
          cout.flush();
          cin >> choice;
          j=0;
          for (i=0; i<strlen(choice.c_str()); i++)
          { if (isdigit(choice.c_str()[i])) j++; }
        } while (i!=j);

        i = atoi(choice.c_str());
        cout << endl;

        if (i>(piNotList.size()-1) && piNotList.size()>0)
        {
          cout << "Please use range 0-" << piNotList.size()-1 << "." << endl;
        }
        else
        {
          piNot = piNotList[i];

          cout << "Notification for: "
               << piNot.Name() << "\n"
               << "Process instance: "
               << piNot.ProcessInstanceName() << "\n"
               << "Creation Time   : "
               << string(piNot.CreationTime()) << "\n" << endl;
        }
        break;


      case 9:
/*-------------------------------------------------------------------*/
/* Show activity instance notification info                          */
/*-------------------------------------------------------------------*/

        do
        {
          cout << "Please enter notification number: ";
          cout.flush();
          cin >> choice;
          j=0;
          for (i=0; i<strlen(choice.c_str()); i++)
          { if (isdigit(choice.c_str()[i])) j++; }
        } while (i!=j);

        i = atoi(choice.c_str());
        cout << endl;

        if (i>(aiNotList.size()-1) && aiNotList.size()>0)
        {
          cout << "Please use range 0-" << aiNotList.size()-1 << "." << endl;
        }
        else
        {
          aiNot = aiNotList[i];

          cout << "Notification for: "
               << aiNot.Name() << "\n"
               << "Process instance: "
               << aiNot.ProcessInstanceName() << "\n"
               << "Creation Time   : "
               << string(aiNot.CreationTime()) << "\n" << endl;
        }
        break;


      case 11:
/*-------------------------------------------------------------------*/
/* Start a support tool corresponding to a workitem                  */
/*-------------------------------------------------------------------*/

        do
        {
          cout << "Please enter workitem number: ";
          cout.flush();
          cin >> choice;
          j=0;
          for (i=0; i<strlen(choice.c_str()); i++)
          { if (isdigit(choice.c_str()[i])) j++; }
        } while (i!=j);

        i = atoi(choice.c_str());
        cout << endl;

        if (i>(vwList.size()-1) && vwList.size()>0)
        {
          cout << "Please use range 0-" << vwList.size()-1 << "." << endl;
        }
        else
        {
          // Clear string vector
          info.erase(info.begin(),info.end());

          workitem = vwList[i];

          if (!workitem.SupportToolsIsNull())
          {
            workitem.SupportTools(info);
            cout << endl;
            cout << "#   Support tool" << endl;
            cout << "----------------" << endl;

            for (j=0; j<info.size(); j++)
            {
             if (j<10) cout << j << "   " << info[j];
             else      cout << j << "  " << info[j];
            }

            do
            {
              cout << "\n\nPlease enter support tool number: ";
              cout.flush();
              cin >> choice;
              j=0;
              for (i=0; i<strlen(choice.c_str()); i++)
              { if (isdigit(choice.c_str()[i])) j++; }
            } while (i!=j);

            i = atoi(choice.c_str());
            cout << endl;

            if (i>(info.size()-1))
            {
              cout << "Please use range 0-" << info.size()-1 << "." << endl;
            }
            else
            {
              cout << "Starting support tool '" << info[i] << "'." << endl;
              rc = workitem.StartTool(info[i]);

              if (rc != FMC_OK)
              {
                cout << "Starting support tool failed with RC " << rc << " - "
                     << result->MessageText() << endl;
              }
            }
          }
          else cout << "\nNo support tools available for this workitem." << endl;
        }
        break;


      case 12:
/*-------------------------------------------------------------------*/
/* Delete a workitem                                                 */
/*-------------------------------------------------------------------*/

        do
        {
          cout << "Please enter workitem number: ";
          cout.flush();
          cin >> choice;
          j=0;
          for (i=0; i<strlen(choice.c_str()); i++)
          { if (isdigit(choice.c_str()[i])) j++; }
        } while (i!=j);

        i = atoi(choice.c_str());
        cout << endl;

        if (i>(vwList.size()-1) && vwList.size()>0)
        {
          cout << "Please use range 0-" << vwList.size()-1 << "." << endl;
        }
        else
        {
          // Clear string vector
          info.erase(info.begin(),info.end());

          workitem = vwList[i];
          rc = workitem.Delete();

          if (rc != FMC_OK)
          {
            cout << "Deleting workitem failed with RC " << rc << " - "
                 << result->MessageText() << endl;
          }
        }
        break;

    }


  } while ( (usrchoice!=255) );


/*-------------------------------------------------------------------*/
/* Shutdown PEA, logoff and deinit the API environment.              */
/*-------------------------------------------------------------------*/

  if (PEAstarted)
  {
    cout << "\nShutting down PEA." << endl;
    rc = service.PEAShutDown();
  }
  cout << "\nLogging off." << endl;
  rc = service.Logoff();
  cout << "Deinitializing API environment." << endl;
  rc = FmcjGlobal::Disconnect();
  delete(result);
  return 0;
}
