/*BEGIN COPYRIGHT */
/**************************************************************************
*
*   Licensed Materials - Property of IBM
*   5697-FM3 5655-A96
*   (C) Copyright IBM Corp. 1998, 2000
*   US Government Users Restricted Rights - Use, duplication or
*   disclosure restricted by GSA ADP Schedule Contract
*   with IBM Corp.
*
**************************************************************************/
/*END COPYRIGHT */

/* BEGIN DISCLAIMER */
/**********************************************************************
*  DISCLAIMER
*  ----------
*  This material contains programming source code for your consideration.
*  These examples have not been thoroughly tested under all conditions.
*  IBM, therefore, cannot guarantee or imply reliability, serviceability,
*  or function of these programs.
*  ALL PROGRAMS CONTAINED HEREIN ARE PROVIDED TO YOU "AS IS", WITHOUT ANY
*  WARRANTIES (EXPRESS OR IMPLIED) OR SUPPORT WHATSOEVER, INCLUDING BUT
*  NOT LIMITED TO ANY IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS
*  FOR A PARTICULAR PURPOSE.
**********************************************************************/
/* END DISCLAIMER */

/**********************************************************************/
/* MQSeries Workflow API Examples - Basic functions                   */
/*                                                                    */
/* This sample code shows how to establish a session to a Workflow    */
/* server and query details about the logged-on user.                 */
/* Additionally, it can be used to change a user's password, handle   */
/* its substitute and toggle its absence status.                      */
/**********************************************************************/

#include <fmcjcrun.h>                   /* MQWF C API interface       */
#include <ctype.h>                      /* character classification   */
#include <string.h>                     /* string conversion           */
#include <stdio.h>                      /* for standard in-output      */
#define INFOSIZE 4097

int main (int argc, char ** argv)
{
  FmcjExecutionServiceHandle    service                 = 0;
  FmcjResultHandle              result                  = 0;
  FmcjPersonHandle              loggedOnPerson          = 0;
  FmcjStringVectorHandle        personRoles             = 0;

  enum FmcjServiceSessionMode   sessionMode             = Fmc_SM_Default;

  char           aRole          [FMC_ROLE_NAME_LENGTH]         = "";
  char           systemGroupName[FMC_SYSTEM_GROUP_NAME_LENGTH] = "";
  char           systemName     [FMC_SYSTEM_NAME_LENGTH]       = "";
  char           user           [FMC_USER_ID_LENGTH]           = "";
  char           password       [FMC_PASSWORD_LENGTH]          = "";
  char           pw             [FMC_PASSWORD_LENGTH]          = "";
  char           pwCheck        [FMC_PASSWORD_LENGTH]          = "";
  char           choice         [1]                            = "";
  char           tInfo          [INFOSIZE]                     = "";

  unsigned long  c                       = 0;
  int            usrchoice               = 0;

  unsigned long  vectSize                = 0;

  APIRET         rc                      = 0;

/*--------------------------------------------------------------------*/
/* Check command line parameters                                      */
/*--------------------------------------------------------------------*/
  if ( (argc<3) || (argc>6) )
  {
    printf("\nUsage: %s %s", argv[0],
           " [-p|-pf] <UserID> <Password> [System Group] [System Name]\n\n");
    printf("       -p : Session mode 'present'\n");
    printf("       -pf: Force logon for session mode 'present'\n");
    fflush(stdout);
    return -1;
  }



  if (strncmp(argv[1],"-",1)==0)   /* Check for session mode option */
  {
    if (strncmp(argv[1],"-pf",3)==0)
      sessionMode=Fmc_SM_PresentHere;
    else
      if (strncmp(argv[1],"-p",2)==0)
        sessionMode=Fmc_SM_Present;
       else
         printf("\nIgnoring unknown option: %s\n", argv[1]);
         fflush(stdout);

    strcpy(user, argv[2]);
    strcpy(password, argv[3]);
    if (argc > 4)
      strcpy(systemGroupName, argv[4]);
    if (argc > 5)
      strcpy(systemName, argv[5]);
  }
  else                             /* Use default session mode */
  {
    strcpy(user, argv[1]);
    strcpy(password, argv[2]);
    if (argc > 3)
      strcpy(systemGroupName, argv[3]);
    if (argc > 4)
      strcpy(systemName, argv[4]);
  }

/*-------------------------------------------------------------------*/
/* Init API environment. Invoke FmcjGlobalConnect() in every thread  */
/* before the server is called.                                      */
/*-------------------------------------------------------------------*/
  rc = FmcjGlobalConnect();

/*-------------------------------------------------------------------*/
/* Fetch result object for current thread                            */
/* Allocate ExecutionService for the given system parameters         */
/*-------------------------------------------------------------------*/
  result = FmcjResultObjectOfCurrentThread();

  /* Choose the correct constructor depending on whether the user has*/
  /* specified system name/system group or not.                      */

  if (strlen(systemName) == 0)
    if (strlen(systemGroupName) == 0)
      rc = FmcjExecutionServiceAllocate(&service); /* Take values from profile */
    else
      rc = FmcjExecutionServiceAllocateForGroup(systemGroupName, &service);
  else
    rc = FmcjExecutionServiceAllocateForSystem(systemName, systemGroupName, &service);

  if (rc != FMC_OK)
  {
    printf("Service object could not be allocated: %u\n",rc);
    FmcjGlobalDisconnect();
    return -1;
  }

/*-------------------------------------------------------------------*/
/* Logon to the specified system. The 'Default' session mode allows  */
/* multiple logons of the same user, but prevents from participation */
/* in workitem pushing. This is only possible in session mode        */
/* 'Present'. A user can have only one 'Present' session. The        */
/* parameter 'PresentHere' is used to force the closing of the       */
/* user's previous 'Present' session to allow opening a new one.     */
/*-------------------------------------------------------------------*/
  rc = FmcjExecutionServiceLogon(service, user, password,
                                 sessionMode, Fmc_SA_NotSet);

  if (rc != FMC_OK)
  {
    printf("Logon failed with RC  %u %s %s\n",rc," - ",
                    FmcjResultMessageText(result, tInfo, INFOSIZE));
    fflush(stdout);
    /* Deinit API environment */
    FmcjExecutionServiceDeallocate( &service );
    FmcjGlobalDisconnect();
    return rc;
  }

  printf("\nLogged on with UserID '%s' ",
             FmcjExecutionServiceUserID(service, tInfo, INFOSIZE));
  printf("on system '%s' ",
             FmcjExecutionServiceSystemName(service, tInfo, INFOSIZE));
  printf("at system group '%s'.\n",
             FmcjExecutionServiceSystemGroupName(service, tInfo, INFOSIZE));
  fflush(stdout);

  printf("Timeout value: %u ms\n", FmcjServiceTimeout(service));
  fflush(stdout);
  printf("Session mode : ");
  switch (sessionMode)
  {
    case Fmc_SM_Default: printf("Default\n"); break;
    case Fmc_SM_Present: printf("Present\n"); break;
    case Fmc_SM_PresentHere: printf("PresentHere\n"); break;
  }
  fflush(stdout);


/*-------------------------------------------------------------------*/
/* Query information about the logged-on user.                       */
/*-------------------------------------------------------------------*/


  /* Fetch information about currently logged-on user */

  rc = FmcjServiceUserSettings(service, &loggedOnPerson);
  if (rc != FMC_OK)
  {
    printf("Getting user data failed with RC %u-%s \n",rc,
                    FmcjResultMessageText(result, tInfo, INFOSIZE));
    fflush(stdout);

    FmcjPersonDeallocate(&loggedOnPerson);
    FmcjExecutionServiceLogoff(service);
    /* Deinit API environment */
    rc = FmcjExecutionServiceDeallocate(&service);
    FmcjGlobalDisconnect();

    return rc;
  }

  FmcjPersonRefresh(loggedOnPerson);

  printf("\nSome information about logged-on user '%s':\n", user);

  printf("     First name        : %s\n",
               FmcjPersonFirstName(loggedOnPerson, tInfo, INFOSIZE));
  printf("     Last  name        : %s\n",
            FmcjPersonLastName(loggedOnPerson, tInfo, INFOSIZE));
  printf("     Level             : %s\n",
            FmcjPersonLevel(loggedOnPerson));
  printf("     Organization      : %s\n",
            FmcjPersonOrganizationName(loggedOnPerson, tInfo, INFOSIZE));
  printf("     Administrator flag: %d\n",
            FmcjPersonIsAdministrator(loggedOnPerson));
  printf("     Manager flag      : %d\n",
            FmcjPersonIsManager(loggedOnPerson));

  personRoles = FmcjPersonNamesOfRoles(loggedOnPerson);

  printf("     Member of roles   : ");
  fflush(stdout);

  vectSize = FmcjStringVectorSize(personRoles);
  for (c=0; c < vectSize; c++)
  {
    FmcjStringVectorNextElement(personRoles, aRole, FMC_ROLE_NAME_LENGTH);
    if (c!=(vectSize-1)) printf("%s, ", aRole);
    else                 printf("%s\n", aRole);

    fflush(stdout);
  }

  if (FmcjPersonSubstituteIsNull(loggedOnPerson))
       printf("     Substitute        : <not set>\n");
  else printf("     Substitute        : %s\n",
              FmcjPersonSubstitute(loggedOnPerson, tInfo, INFOSIZE));
  printf("     Absence flag      : %d\n",
              FmcjPersonIsAbsent(loggedOnPerson));
  printf("     IsResetAbsence    : %d\n",
              FmcjPersonIsResetAbsence(loggedOnPerson));
  fflush(stdout);

  /* Clean up the string vector */
  FmcjStringVectorDeallocate(&personRoles);

/*-------------------------------------------------------------------*/
/* Provide supported actions                                         */
/*-------------------------------------------------------------------*/
  printf("\nCommands: [p] Change user's password            [a] Toggle user's absence flag\n");
  printf("          [s] Set a substitute for this user    [q] Quit program\n");
  printf("[p/s/a/q]? ");
  fflush(stdout);

  fgets(choice,3,stdin);

  if (strncmp(choice,"p",1)==0) usrchoice=1;
  if (strncmp(choice,"s",1)==0) usrchoice=2;
  if (strncmp(choice,"a",1)==0) usrchoice=3;

  switch (usrchoice)
  {
    case 1:
/*-------------------------------------------------------------------*/
/* Change the password for the logged-on user.                       */
/*-------------------------------------------------------------------*/

      printf("\nEnter new password             : "); fflush(stdout);
      scanf("%32[^\n]",pw);
      fgets(tInfo,INFOSIZE,stdin); /* clear the stdin */

      printf("Enter it again for verification: "); fflush(stdout);
      scanf("%32[^\n]",pwCheck);
      fgets(tInfo,INFOSIZE,stdin); /* clear the stdin */

      if (!strcmp(pw,pwCheck))
      {
        printf("\nChanging password.\n");
        fflush(stdout);
        rc = FmcjServiceSetPassword(service, pw);
        if (rc != FMC_OK)
        {
          printf("Setting password failed with RC  %u-%s\n",
                   rc, FmcjResultMessageText(result, tInfo, INFOSIZE));
        }
      }
      else
      {
        printf("\nPasswords do not match.\n");
      }

      break;


    case 2:
/*-------------------------------------------------------------------*/
/* Set a substitute for the logged-on user.                          */
/*-------------------------------------------------------------------*/
      printf("\nEnter substitute user name: ");
      fflush(stdout);

      scanf("%48[^\n]",tInfo);
      rc = FmcjPersonSetSubstitute(loggedOnPerson, tInfo);


      if (rc != FMC_OK)
      {
        printf("Setting substitute failed with RC  %u-%s\n",
           rc, FmcjResultMessageText(result, tInfo, INFOSIZE));
      }
      else printf("Substitute successfully set to %s\n", tInfo);

      fgets(tInfo,INFOSIZE,stdin); /* clear the stdin  */
      break;


    case 3:
/*-------------------------------------------------------------------*/
/* Toggle the user's absence flag                                    */
/* The flag may be reset during next logon depending on the person's */
/* reset absence setting.                                            */
/*-------------------------------------------------------------------*/
      if (FmcjPersonIsAbsent(loggedOnPerson))
          rc = FmcjPersonSetAbsence(loggedOnPerson, false);
      else
          rc = FmcjPersonSetAbsence(loggedOnPerson, true);

      if (rc != FMC_OK)
      {
        printf("Toggling absence flag failed with RC  %u-%s\n",
             rc, FmcjResultMessageText(result, tInfo, INFOSIZE));
      }
      else
        printf("Absence flag set to %d\n",
               FmcjPersonIsAbsent(loggedOnPerson));

      break;

  }  /* end switch */

  FmcjPersonDeallocate(&loggedOnPerson);

/*-------------------------------------------------------------------*/
/* Logoff and deinit the API environment.                            */
/*-------------------------------------------------------------------*/

  printf("\nLogging off.\n");
  rc = FmcjExecutionServiceLogoff(service);

  rc= FmcjExecutionServiceDeallocate( &service );

  printf("Deinitializing API environment.\n");
  rc = FmcjGlobalDisconnect();

  return 0;
}
