/*BEGIN COPYRIGHT */
/**************************************************************************
*
*   Licensed Materials - Property of IBM
*   5697-FM3 5655-A96
*   (C) Copyright IBM Corp. 1998, 2000
*   US Government Users Restricted Rights - Use, duplication or
*   disclosure restricted by GSA ADP Schedule Contract
*   with IBM Corp.
*
**************************************************************************/
/*END COPYRIGHT */

/* BEGIN DISCLAIMER */
/**********************************************************************
*  DISCLAIMER
*  ----------
*  This material contains programming source code for your consideration.
*  These examples have not been thoroughly tested under all conditions.
*  IBM, therefore, cannot guarantee or imply reliability, serviceability,
*  or function of these programs.
*  ALL PROGRAMS CONTAINED HEREIN ARE PROVIDED TO YOU "AS IS", WITHOUT ANY
*  WARRANTIES (EXPRESS OR IMPLIED) OR SUPPORT WHATSOEVER, INCLUDING BUT
*  NOT LIMITED TO ANY IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS
*  FOR A PARTICULAR PURPOSE.
**********************************************************************/
/* END DISCLAIMER */

/**********************************************************************/
/* MQSeries Workflow API Examples - Activity implemenation            */
/*                                                                    */
/* Example activity implementation program which can be used to       */
/* analyze unknown container structures and set member values. The    */
/* return code of the program is specified via a command line switch. */
/**********************************************************************/

// Import necessary MQWF Java API Interface

import com.ibm.workflow.api.*;

// Various needed classes

import java.io.*;
import java.util.*;

public class ActivityImplementation {

public static PrintWriter out;

public static void main(String args[])
{

  try { out = new PrintWriter(new BufferedWriter(new FileWriter("ActivityImplementation.log"))); }
  catch (IOException e) { }

  ReadWriteContainer outctnr;

  // Maximum nesting depth is 10
  ContainerElement[][] members = new ContainerElement[10][];

  // Maximum element number is 200
  ContainerElement[][] leaves = new ContainerElement[200][];

  ContainerElement     element = null;

  int ulMemberCount           = 0;

  String membername;
  String membertype;

  String container = "";

  String strvalbuf;
  byte[] binvalbuf = { 0,1,2,3,4,5,6,7,8,9,10 };
  int lvalbuf        = 0;
  double fvalbuf        = 0.0;

  String tInfo;
  int j = 0, k = 0, l = 0;
  int index = 0;
  Calendar ltime = Calendar.getInstance();

  int param = -1;
  int pimreturn = 0;

  String setstr ="";
  int setlong = 0;
  double setdbl = 0;

  ltime.setTime(new Date());

  BufferedReader in = new BufferedReader(new InputStreamReader(System.in));
  boolean again = false;
  String choice;


/*--------------------------------------------------------------------*/
/* Check command line parameters                                      */
/*--------------------------------------------------------------------*/

  if (args.length!=1)
  {
    System.out.println("\nUsage: java ActivityImplementation <-r{0|1|2|3|4|5}>\n"
                       + "       -rx: x specifies the return code of the program\n\n"
                       + "       Program output is written to ActivityImplementation.log\n");
    System.exit(-1);
  }

  try { // general try clause for whole example

  Agent agent = new Agent();
  agent.setLocator(Agent.LOC_LOCATOR);
  agent.setName("LOCAL");

  ExecutionService service = agent.locate("","");
  System.out.println();

  if (args[0].equals("-r0")) param=0;
  if (args[0].equals("-r1")) param=1;
  if (args[0].equals("-r2")) param=2;
  if (args[0].equals("-r3")) param=3;
  if (args[0].equals("-r4")) param=4;
  if (args[0].equals("-r5")) param=5;

  switch (param)
  {
   case -1:
     System.out.println("\nUsage: java ActivityImplementation <-r{0|1|2|3|4|5}>\n"
                       + "       -rx: x specifies the return code of the program\n\n"
                       + "       Program output is written to ActivityImplementation.log\n");
     System.exit(-1);
     break;
   case 0: pimreturn = 0; break;
   case 1: pimreturn = 1; break;
   case 2: pimreturn = 2; break;
   case 3: pimreturn = 3; break;
   case 4: pimreturn = 4; break;
   case 5: pimreturn = 5; break;
  }


  out.write("\n\n" + "-------------------------------------------------------\n"
        + "API Tutorial - Activity implementation example program.\n"
        + "Called at : " + ltime.getTime().toString() + "\n"
        + "Desired RC: " + pimreturn + "\n\n");


/*-------------------------------------------------------------------*/
/* Establish a Passthrough session to the specified system and       */
/* system group. Note: Passthrough is not absolutely necessary to    */
/* handle the input and output containers only. In this sample, it   */
/* is merely used to add information about the logged-on user to the */
/* log file.                                                         */
/*-------------------------------------------------------------------*/

  // Only establish a passthrough session

  service.passthrough();

  out.write("Logged on with UserID '" + service.userID()
        + "' on system '" + service.systemName()
        + "' at system group '" + service.systemGroupName()
        + "'.\n" + "Timeout value: " + service.timeout() + "\n");
  System.out.println("Logged on with UserID '" + service.userID()
        + "' on system '" + service.systemName()
        + "' at system group '" + service.systemGroupName()
        + "'.\n" + "Timeout value: " + service.timeout());


/************************************************************************/
/* First we cope with the activity's INPUT container:                   */
/*  - get the container                                                 */
/*  - get the leaves of the container                                   */
/*  - display the member values                                         */
/************************************************************************/

/*----- Get Input Container --------------------------------------------*/

  ReadOnlyContainer inctnr;
  ExecutionAgent eAgent = agent.getExecutionAgent();

  inctnr = eAgent.inContainer();

  out.write("\nReceived input container '" + inctnr.type() + "'\n");
  System.out.println("\nReceived input container '" + inctnr.type() + "'\n");

/*----- Get Leave Count ------------------------------------------------*/

  int ulLeafCount = inctnr.allLeafCount();

  out.write("Input container AllLeafCount() =  " + ulLeafCount + "\n");

/*----- Get the Leaves -------------------------------------------------*/

  leaves[0]=inctnr.allLeaves();

  if (ulLeafCount != leaves[0].length)
  {
    out.write("LeafCount - vector size mismatch: "
          + "ulLeafCount = " + ulLeafCount
          + " size = "       + leaves[0].length + "\n");
  }

/*----- Show the data members ------------------------------------------*/

  out.write("Input container leaves:\n");

  element = leaves[0][0];

  for (j=0; j < ulLeafCount; j++)
  {
    membername = element.fullName();
    membertype = element.type();

    if (membertype.equals("STRING"))
    {
      try
      {
        strvalbuf = element.getString();
        out.write("STRING '" + membername + "'= " + strvalbuf  + "\n");
      }
      catch (FmcException e)
      {
       switch (e.rc)
       {
         case FmcException.FMC_ERROR_MEMBER_NOT_SET:
           out.write("STRING '" + membername  + "'= <not set>" + "\n");
           break;
         default:
           out.write("Failed to access " + membername + " with RC " + e.rc + "\n");
           break;
       }
      }
    } /* end if STRING */

    if (membertype.equals("LONG"))
    {
      try
      {
        lvalbuf = element.getLong();
        out.write("LONG '" + membername + "'= " + lvalbuf  + "\n");
      }
      catch (FmcException e)
      {
       switch (e.rc)
       {
         case FmcException.FMC_ERROR_MEMBER_NOT_SET:
           out.write("LONG '" + membername  + "'= <not set>" + "\n");
           break;
         default:
           out.write("Failed to access " + membername + " with RC " + e.rc + "\n");
           break;
       }
      }
    } /* end if LONG */

    if (membertype.equals("FLOAT"))
    {
      try
      {
        fvalbuf = element.getDouble();
        out.write("FLOAT '" + membername + "'= " + fvalbuf  + "\n");
      }
      catch (FmcException e)
      {
       switch (e.rc)
       {
         case FmcException.FMC_ERROR_MEMBER_NOT_SET:
           out.write("FLOAT '" + membername  + "'= <not set>" + "\n");
           break;
         default:
           out.write("Failed to access " + membername + " with RC " + e.rc + "\n");
           break;
       }
      }
    } /* end if FLOAT */

    if (membertype.equals("BINARY"))
    {
      try
      {
        binvalbuf = element.getBuffer();
        out.write("BINARY '" + membername + "'= <not shown>\n");
      }
      catch (FmcException e)
      {
       switch (e.rc)
       {
         case FmcException.FMC_ERROR_MEMBER_NOT_SET:
           out.write("BINARY '" + membername  + "'= <not set>" + "\n");
           break;
         default:
           out.write("Failed to access " + membername + " with RC " + e.rc + "\n");
           break;
       }
      }
     } /* end if BINARY */

    element = leaves[0][j];

  } /* end for */


/************************************************************************/
/* Now we process the activity's OUTPUT container, but only if the      */
/* program was not started with the -s switch                           */
/*  - get the container                                                 */
/*  - navigate through the ContainerElement levels to the leaves        */
/*  - modify the member values                                          */
/*  - send the container to the server                                  */
/************************************************************************/

/*----- Get Output Container -------------------------------------------*/

  outctnr = eAgent.outContainer();

  out.write("Received output container '" + outctnr.type() + "'\n");
  System.out.println("Received output container '" + outctnr.type() + "'");


/*----- Navigate through the ContainerElement structures ---------------*/

  int level            = 0;  // Current nesting level
  int start            = 0;  // Start "stack pointer" of vectors
  int current          = 0;  // End "stack pointer" of vectors
  boolean AllLeavesReached = false;

  // Get the number of 1st level container elements

  ulMemberCount = outctnr.memberCount();

  if (ulMemberCount == 0)       // No data structure at all, error?
  {
    AllLeavesReached = true;
  }
  else                          // Get the vector of 1st level container elements
  {
    members[level] = outctnr.structMembers();
  }

  while (AllLeavesReached == false)
  {
    out.write("Number of members: " + ulMemberCount + "\n");

    // Check consistency

    if (members[level].length != ulMemberCount)
    {
      out.write("MemberCount - Vector Size mismatch: "
            + "ulMemberCount = " + ulMemberCount
            + " Size = "         + members[level].length + "\n");
    }

    element = members[level][0];

    out.write("IsStruct-IsLeaf-IsArray-Cardinality-Membername\n");

    for (j=0; j < ulMemberCount; j++)
    {
      out.write("       " + element.isStruct());

      if (element.isStruct())
      {
        // Put the next nesting level of this data structure "on the stack"
        leaves[current] = element.structMembers();
        current++;
      }

      out.write("      " + element.isLeaf() + "       " + element.isArray()
            + "           " + element.cardinality() + " "
            + element.fullName() + "\n");

      element = members[level][j];

    } /* end for */

    if (start >= current)        // Nothing "on the stack"
    {
      AllLeavesReached = true;
    }
    else
    {
      level++;
      // Get the next vector from the stack and increase stack pointer
      members[level] = leaves[start];
      start++;
      ulMemberCount = members[level].length;
    }

  } /* end while */

/*----- Modify the data members ----------------------------------------*/

  out.write("\n\nSetting the data members...\n");
  System.out.println("\nSetting the data members...\n");

  leaves[0] = outctnr.leaves();

  ulLeafCount = leaves[0].length;

  try
  {

    for (j=0; j < ulLeafCount; j++)
    {
      element = leaves[0][j];

      membername = element.fullName();
      membertype = element.type();

      if (membertype.equals("STRING"))
      {
        if ( membername.endsWith("]") || membername.endsWith(")") )

        // Array element, take the array function
        {
               // Remember the index and remove the suffix from the name

          if (membername.endsWith("]"))
          {
            try { index = Integer.parseInt(membername.substring(membername.indexOf("[")+1,membername.indexOf("]")-1)); }
            catch (NumberFormatException e) {}
          }
          else
          {
            try { index = Integer.parseInt(membername.substring(membername.indexOf("(")+1,membername.indexOf(")")-1)); }
            catch (NumberFormatException e) {}
          }

          System.out.print("Enter a value for STRING array member '"
               + membername + "[" + index + "]'? [y/n] ");

          setstr = in.readLine();

          if (setstr.equalsIgnoreCase("y"))
          {
            System.out.print("Value: ");
            setstr = in.readLine();

            outctnr.setString2(membername,index,setstr);
            strvalbuf = outctnr.getString2(membername,index);

            out.write("Setting ArrayStringValue '" + membername + "[" + index + "]' = " + strvalbuf + "\n");
            System.out.println("Setting ArrayStringValue '" + membername + "[" + index + "]' = " + strvalbuf + "\n");

          }
        }
        else
        {

          System.out.print("Enter a value for STRING member '"
               + membername + "'? [y/n] ");
          setstr = in.readLine();

          if (setstr.equalsIgnoreCase("y"))
          {
            System.out.print("Value: ");
            setstr = in.readLine();

            outctnr.setString(membername,setstr);
            strvalbuf = outctnr.getString(membername);

            out.write("Setting StringValue '" + membername + "'= " + strvalbuf + "\n");
            System.out.println("Setting StringValue '" + membername + "'= " + strvalbuf + "\n");
          }
        }
      } /* end if STRING */


      if (membertype.equals("LONG"))
      {
        if ( membername.endsWith("]") || membername.endsWith(")") )

        // Array element, take the array function
        {
          // Remember the index and remove the suffix from the name

          if (membername.endsWith("]"))
          {
            try { index = Integer.parseInt(membername.substring(membername.indexOf("[")+1,membername.indexOf("]")-1)); }
            catch (NumberFormatException e) {}
          }
          else
          {
            try { index = Integer.parseInt(membername.substring(membername.indexOf("(")+1,membername.indexOf(")")-1)); }
            catch (NumberFormatException e) {}
          }

          System.out.print("Enter a value for LONG array member '"
               + membername + "[" + index + "]'? [y/n] ");

          setstr = in.readLine();

          if (setstr.equalsIgnoreCase("y"))
          {
            do
            {
              again=false;
              System.out.print("\nValue: ");
              choice = in.readLine();
              try { setlong = Integer.parseInt(choice); }
              catch (NumberFormatException e) { again=true; }
               } while (again);

            outctnr.setLong2(membername,index,setlong);
            lvalbuf = outctnr.getLong2(membername,index);

            out.write("Setting ArrayLongValue '" + membername + "[" + index + "]' = " + lvalbuf + "\n");
            System.out.println("Setting ArrayLongValue '" + membername + "[" + index + "]' = " + lvalbuf + "\n");

          }
        }
        else
        {

          System.out.print("Enter a value for LONG member '"
               + membername + "'? [y/n] ");

          setstr = in.readLine();

          if (setstr.equalsIgnoreCase("y"))
          {
            do
            {
              again=false;
              System.out.print("\nValue: ");
              choice = in.readLine();
              try { setlong = Integer.parseInt(choice); }
              catch (NumberFormatException e) { again=true; }
               } while (again);

            outctnr.setLong(membername,setlong);
            lvalbuf = outctnr.getLong(membername);

            out.write("Setting LongValue '" + membername + "' = " + lvalbuf + "\n");
            System.out.println("Setting LongValue '" + membername + "' = " + lvalbuf + "\n");
          }
        }
      } /* end if LONG */


      if (membertype.equals("FLOAT"))
      {
        if ( membername.endsWith("]") || membername.endsWith(")") )

        // Array element, take the array function
        {
               // Remember the index and remove the suffix from the name

          if (membername.endsWith("]"))
          {
            try { index = Integer.parseInt(membername.substring(membername.indexOf("[")+1,membername.indexOf("]")-1)); }
            catch (NumberFormatException e) {}
          }
          else
          {
            try { index = Integer.parseInt(membername.substring(membername.indexOf("(")+1,membername.indexOf(")")-1)); }
            catch (NumberFormatException e) {}
          }

          System.out.print("Enter a value for FLOAT array member '"
               + membername + "[" + index + "]'? [y/n] ");

          setstr = in.readLine();

          if (setstr.equalsIgnoreCase("y"))
          {
            do
            {
              again=false;
              System.out.print("\nValue: ");
              choice = in.readLine();
              try { setdbl = (Double.valueOf(choice)).doubleValue(); }
              catch (NumberFormatException e) { again=true; }
               } while (again);

            outctnr.setDouble2(membername,index,setdbl);
            fvalbuf = outctnr.getDouble2(membername,index);

            out.write("Setting ArrayFloatValue '" + membername + "[" + index + "]' = " + fvalbuf + "\n");
            System.out.println("Setting ArrayFloatValue '" + membername + "[" + index + "]' = " + fvalbuf + "\n");

          }
        }
        else
        {
          System.out.print("Enter a value for FLOAT member '"
               + membername + "'? [y/n] ");

          setstr = in.readLine();

          if (setstr.equalsIgnoreCase("y"))
          {
            do
            {
              again=false;
              System.out.print("\nValue: ");
              choice = in.readLine();
              try { setdbl = (Double.valueOf(choice)).doubleValue(); }
              catch (NumberFormatException e) { again=true; }
               } while (again);

            outctnr.setDouble(membername,setdbl);
            fvalbuf = outctnr.getDouble(membername);

            out.write("Setting FloatValue '" + membername + "' = " + fvalbuf + "\n");
            System.out.println("Setting FloatValue '" + membername + "' = " + fvalbuf + "\n");
          }
        }
      } /* end if FLOAT */


      if (membertype.equals("BINARY"))
      {
        if ( membername.endsWith("]") || membername.endsWith(")") )

        // Array element, take the array function
        {
               // Remember the index and remove the suffix from the name

          if (membername.endsWith("]"))
          {
            try { index = Integer.parseInt(membername.substring(membername.indexOf("[")+1,membername.indexOf("]")-1)); }
            catch (NumberFormatException e) {}
          }
          else
          {
            try { index = Integer.parseInt(membername.substring(membername.indexOf("(")+1,membername.indexOf(")")-1)); }
            catch (NumberFormatException e) {}
          }

          outctnr.setBuffer2(membername,index,binvalbuf);
          binvalbuf = outctnr.getBuffer2(membername,index);

          out.write("Setting ArrayBinaryValue '" + membername + "[" + index + "]' = <not shown>\n");
          System.out.println("Setting ArrayBinaryValue '" + membername + "[" + index + "]' = <not shown>\n");
        }
        else
        {
          outctnr.setBuffer(membername,binvalbuf);
          binvalbuf = outctnr.getBuffer(membername);

          out.write("Setting BinaryValue '" + membername + "' = <not shown>\n");
          System.out.println("Setting BinaryValue '" + membername + "' = <not shown>\n");
        }
      } /* end if BINARY */

    } /* end for */
  }/* End try*/
  catch (FmcException e)
  {
  }/* End catch*/

/*-------------------------------------------------------------------*/
/* Send output container to PEA                                      */
/*-------------------------------------------------------------------*/
  System.out.println("\nSetting output container.");
  out.write("\nSetting output container.\n");
  eAgent.setOutContainer(outctnr);


/*-------------------------------------------------------------------*/
/* Logoff and deinit the API environment.                            */
/*-------------------------------------------------------------------*/

  System.out.println("\nLogging off.");
  out.write("\nLogging off.\n");
  service.logoff();
  out.flush();
  out.close();
  System.exit(pimreturn);

  } // end of general try clause

  catch(FmcException e)
  {
    int c;

    // Catch and report details about the FmcException
    System.out.println("FmcException occured");
    System.out.println("  RC         : " + e.rc);
    System.out.println("  Origin     : " + e.origin);
    System.out.println("  MessageText: " + e.messageText);
    System.out.println("  Exception  : " + e.getMessage());
    System.out.println("  Parameters : ");
    for ( c = 0; c < e.parameters.length ; c++)
    {
      System.out.println("    " + e.parameters[c] );
    }
    System.out.println("  StackTrace : ");
    e.printStackTrace();

    out.write("FmcException occured");
    out.write("\n  RC         : " + e.rc);
    out.write("\n  Origin     : " + e.origin);
    out.write("\n  MessageText: " + e.messageText);
    out.write("\n  Exception  : " + e.getMessage());
    out.write("\n  Parameters : ");
    for ( c = 0; c < e.parameters.length ; c++)
    {
      out.write("    " + e.parameters[c] );
    }
    out.write("\n  StackTrace : ");
    e.printStackTrace(out);
    out.flush();
    out.close();

  }
  catch(Exception e)
  {
    // Catch and report any exception that occurred.
    e.printStackTrace();
    e.printStackTrace(out);
    out.flush();
    out.close();
  }

} // end of void main

} // end of ActivityImplementation

