/*BEGIN COPYRIGHT */
/**************************************************************************
*
*   Licensed Materials - Property of IBM
*   5697-FM3 5655-A96
*   (C) Copyright IBM Corp. 2004
*   US Government Users Restricted Rights - Use, duplication or
*   disclosure restricted by GSA ADP Schedule Contract
*   with IBM Corp.
*
**************************************************************************/
/*END COPYRIGHT */

/* BEGIN DISCLAIMER */
/**********************************************************************
*  DISCLAIMER
*  ----------
*  This material contains programming source code for your consideration.
*  These examples have not been thoroughly tested under all conditions.
*  IBM, therefore, cannot guarantee or imply reliability, serviceability,
*  or function of these programs.
*  ALL PROGRAMS CONTAINED HEREIN ARE PROVIDED TO YOU "AS IS", WITHOUT ANY
*  WARRANTIES (EXPRESS OR IMPLIED) OR SUPPORT WHATSOEVER, INCLUDING BUT
*  NOT LIMITED TO ANY IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS
*  FOR A PARTICULAR PURPOSE.
**********************************************************************/
/* END DISCLAIMER */

/**********************************************************************
* IBM WebSphere MQ Workflow API Tutorial - API calls via EJB         
*                                                                    
* This sample program shows how to call Workflow APIs from an        
* Enterprise Java Bean and pass the result to a client program.    
* 
* Looks up the bean interface, creats a bean instance and calls 
* the following methods provided by the bean:
* 
* logon
* queryTemplates
* createInstance  
* queryInstances
* queryItems
* deleteInstance
* logoff
**********************************************************************/

package com.ibm.workflow.api.smp.EJB.client;

import java.io.FileInputStream;
import java.util.Properties;

//import javax.rmi.PortableRemoteObject;
import javax.naming.Context;
import javax.naming.InitialContext;

import com.ibm.workflow.api.smp.EJB.WorkflowEJB;
import com.ibm.workflow.api.smp.EJB.WorkflowEJBHome;

public class EJBTestClient {

	private static String sessionID = null;
	private static ServiceWrapper es = null;

	private static Properties prop = new Properties();

	/** EJB home */
	private WorkflowEJBHome workflowHome = null;
	private String ejbHomeString = null;
	private static WorkflowEJB bean = null;

	/**
	 * Constructor
	 *
	 * Reads the properties from file, gets the initial JNDI context,
	 * and gets the EJB home interface.
	 */
	public EJBTestClient() {
		System.out.println("MQSeries Workflow EJB Invocation Example ");
		System.out.println(
			"---------------------------------------------------------------");
		System.out.println("Initializing EJBTestClient");

		// Read properties from configuration file
		try {
			prop.load(new FileInputStream("EJBInvocation.properties"));
			prop.put(
				Context.INITIAL_CONTEXT_FACTORY,
				prop.getProperty("EJB.INITIAL_CONTEXT_FACTORY"));
			prop.put(
				Context.PROVIDER_URL,
				prop.getProperty("EJB.PROVIDER_URL"));
			// Create the JNDI naming context
			InitialContext cxt = new InitialContext(prop);

			// Retrieve the EJB home.
			// Temporary needed dummy object
			ejbHomeString = prop.getProperty("EJB.HOME_STRING");
			Object obj = cxt.lookup(ejbHomeString);

			workflowHome =
				(WorkflowEJBHome) javax.rmi.PortableRemoteObject.narrow(
					obj,
					WorkflowEJBHome.class);

			bean = workflowHome.create();
			System.out.println("Bean created ");

			System.out.println("EJBTestClient initialized");
			System.out.println(
				"---------------------------------------------------------------");
		} catch (Exception xcpt) {
			System.out.println("\nException caught:");
			xcpt.printStackTrace();
			System.exit(1);
		}

	}

	/**
	 * Main method, no parameter evaluation
	 */
	public static void main(String[] args) {

		try {
			EJBTestClient myself = new EJBTestClient();

			String userID = prop.getProperty("EJB.UserID");
			String password = prop.getProperty("EJB.Password");
			String system = prop.getProperty("EJB.System");
			String systemGroup = prop.getProperty("EJB.SystemGroup");

			// Logon to MQWF
			String result = bean.logon(userID, password, system, systemGroup);

			if (result.startsWith("error")) {
				System.out.println("Failed to logon: " + result.substring(6));
				System.exit(1);
			} else {
				sessionID = result;
				System.out.println(userID + " logged on.");
				es = new ServiceWrapper(userID, systemGroup, system, sessionID);
			}

			////////////////////////////////////////////////////////////////
			//
			// Now that we are logged on we can call any MQWF API that
			// is provided by the EJB.
			//
			////////////////////////////////////////////////////////////////
			String filter = prop.getProperty("EJB.ProcessTemplateFilter");
			String sort = null;
			Integer threshold = null;

			// Query process templates
			ProcessTemplateWrapper[] templates =
				bean.queryTemplates(es, filter, sort, threshold);

			// Show the retrieved process templates and 
			// create a process instance of each 	
			System.out.println("\nFound the following process templates: ");
			System.out.println("--------------------------------------");
			for (int i = 0; templates != null && i < templates.length; i++) {
				System.out.println(templates[i].name());
				bean.createInstance(es, templates[i].oid());
				System.out.println(
					"\nCreated process instance for template "
						+ templates[i].name());
			}

			filter = null;

			// Query process instances
			ProcessInstanceWrapper[] instances =
				bean.queryInstances(es, filter, sort, threshold);

			// Show the retrieved process instances	
			System.out.println("\nFound the following process instances: ");
			System.out.println("--------------------------------------");
			for (int i = 0; instances != null && i < instances.length; i++) {
				System.out.println(instances[i].name());
			}

			filter = null;

			// Query workitems
			WorkItemWrapper[] items =
				bean.queryItems(es, filter, sort, threshold);

			// Show the retrieved items
			System.out.println("\nFound the following work items: ");
			System.out.println("--------------------------------------");
			for (int i = 0; items != null && i < items.length; i++) {
				System.out.println(items[i].name());
			}

			// Cleanup. Delete the process instances created by me
			System.out.println("\nDeleting process instances: ");
			System.out.println("--------------------------------------");
			for (int i = 0; instances != null && i < instances.length; i++) {
				System.out.println(instances[i].name());
				bean.deleteInstance(es, instances[i].oid());
			}

			// Logoff
			result = bean.logoff(es);
			System.out.println("\n" + userID + " logged off ");

		} catch (Exception xcpt) {
			System.out.println("\nException caught:");
			xcpt.printStackTrace();
			System.exit(1);
		}
	}
}