/*BEGIN COPYRIGHT */
/**************************************************************************
*
*   Licensed Materials - Property of IBM
*   5697-FM3 5655-A96
*   (C) Copyright IBM Corp. 2004
*   US Government Users Restricted Rights - Use, duplication or
*   disclosure restricted by GSA ADP Schedule Contract
*   with IBM Corp.
*
**************************************************************************/
/*END COPYRIGHT */

/* BEGIN DISCLAIMER */
/**********************************************************************
*  DISCLAIMER
*  ----------
*  This material contains programming source code for your consideration.
*  These examples have not been thoroughly tested under all conditions.
*  IBM, therefore, cannot guarantee or imply reliability, serviceability,
*  or function of these programs.
*  ALL PROGRAMS CONTAINED HEREIN ARE PROVIDED TO YOU "AS IS", WITHOUT ANY
*  WARRANTIES (EXPRESS OR IMPLIED) OR SUPPORT WHATSOEVER, INCLUDING BUT
*  NOT LIMITED TO ANY IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS
*  FOR A PARTICULAR PURPOSE.
**********************************************************************/
/* END DISCLAIMER */

/**********************************************************************
* IBM WebSphere MQ Workflow API Tutorial - API calls via EJB         
*                                                                    
* This sample program shows how to call Workflow APIs from an        
* Enterprise Java Bean and pass the result to a client program.      
**********************************************************************/

package com.ibm.workflow.api.smp.EJB;

import java.beans.PropertyVetoException;

// Import necessary MQWF Java API interface classes
import com.ibm.workflow.api.*;
import com.ibm.workflow.api.ServicePackage.*;

// Import wrapper classes to pass result to the client
import com.ibm.workflow.api.smp.EJB.client.ServiceWrapper;
import com.ibm.workflow.api.smp.EJB.client.ProcessTemplateWrapper;
import com.ibm.workflow.api.smp.EJB.client.ProcessInstanceWrapper;
import com.ibm.workflow.api.smp.EJB.client.WorkItemWrapper;

/**
 * Bean implementation class for Enterprise Bean: WorkflowEJB
 */
public class WorkflowEJBBean implements javax.ejb.SessionBean {

	private javax.ejb.SessionContext mySessionCtx;
	// session factory of the MQWF Java API
	private Agent theAgent = null;

	/**
	 * GetSessionContext
	 */
	public javax.ejb.SessionContext getSessionContext() {
		return mySessionCtx;
	}
	/**
	 * SetSessionContext
	 */
	public void setSessionContext(javax.ejb.SessionContext ctx) {
		mySessionCtx = ctx;
	}
	/**
	 * Create MQWF session factory
	 */
	public void ejbCreate() throws javax.ejb.CreateException {
		try {
			if (theAgent == null) {
				theAgent = new Agent();
				theAgent.setLocator(Agent.LOC_LOCATOR);
				theAgent.setName("MQWFAgent");
				VersionInfo version = theAgent.versionInfo();
				if (version.build.indexOf("Native") == -1) {
					System.out.println("You are using the classic Java API.");
					System.out.println("Build " + version.build);
				} else {
					System.out.println("You are using the native Java API.");
					System.out.println("Build " + version.build);
				}
			}
		} catch (java.beans.PropertyVetoException ex) {
			System.out.println("Exception occured: " + ex.getMessage());
			throw new javax.ejb.CreateException("error" + ex.getMessage());
		} catch (FmcException ex) {
			System.out.println("Exception occured: " + ex.getMessage());
			throw new javax.ejb.CreateException("error" + ex.getMessage());
		}

	}
	/**
	 * EjbActivate
	 */
	public void ejbActivate() {
	}
	/**
	 * EjbPassivate
	 */
	public void ejbPassivate() {
	}
	/**
	 * EjbRemove
	 */
	public void ejbRemove() {
	}

	/**
	 * Logon to workflow server
	 * @param userID MQWF userID
	 * @param password password
	 * @param system system name,an be null
	 * @param systemGroup system group name, can be null
	 * @return sessionID enables succeeding stateless API calls
	 */
	public String logon(
		String userID,
		String password,
		String system,
		String systemGroup) {

		String sessionID = "";
		ExecutionService service = null;
		try {

			if (systemGroup.equals(null)) {
				systemGroup = "";
			}
			if (system.equals(null)) {
				system = "";
			}

			service = theAgent.locate(systemGroup, system);
			System.out.println("Logon(" + userID + ")");
			service.logon2(
				userID,
				password,
				SessionMode.DEFAULT,
				AbsenceIndicator.LEAVE);
			sessionID = service.sessionID();

		} catch (FmcException ex) {
			System.out.println("Exception occured: " + ex.getMessage());
			return "error" + ex.getMessage();
		}
		return sessionID;
	}

	/**
	 * Logoff from server
	 * @param service session rep to re-create session
	 */
	public String logoff(ServiceWrapper service) {
		String userID = service.user();
		String sessionID = service.id();
		String system = service.system();
		String systemGroup = service.group();

		String response = "";
		try {
			// Re-establish session
			ExecutionService serv = theAgent.locate(systemGroup, system);
			serv.setSessionContext(userID, sessionID);
			
			serv.logoff();
			System.out.println("Logoff(" + userID + ")");
		} catch (FmcException ex) {
			System.out.println("Exception occured: " + ex.getMessage());
			return "error" + ex.getMessage();
		}
		return response;
	}

	/**
	 * Query process templates
	 * @param service session rep to re-create session
	 * @param filter filter string
	 * @param sort sort string
	 * @param threshold threshold
	 * @return array of process template reps
	 */
	public ProcessTemplateWrapper[] queryTemplates(
		ServiceWrapper service,
		String filter,
		String sort,
		Integer threshold) {
		ProcessTemplate[] templates = null;
		ProcessTemplateWrapper[] result = null;

		try {
			// Re-establish session
			ExecutionService serv =
				theAgent.locate(service.group(), service.system());
			serv.setSessionContext(service.user(), service.id());
			
			templates = serv.queryProcessTemplates(filter, sort, threshold);
			if (templates != null) {
				result = new ProcessTemplateWrapper[templates.length];
				for (int i = 0; i < templates.length; i++) {
					result[i] =
						new ProcessTemplateWrapper(
							templates[i].persistentOid(),
							templates[i].name());
				}
			}

		} catch (FmcException ex) {
			System.out.println("Exception occured: " + ex.getMessage());
			// return "error" + ex.getMessage();
			return null;
		}
		return result;
	}

	/**
	 * Query process instances
	 * @param service session rep to re-create session
	 * @param filter filter string
	 * @param sort sort string
	 * @param threshold threshold
	 * @return array of process instance reps
	 */
	public ProcessInstanceWrapper[] queryInstances(
		ServiceWrapper service,
		String filter,
		String sort,
		Integer threshold) {
		ProcessInstance[] instances = null;
		ProcessInstanceWrapper[] result = null;
		try {
			// Re-establish session
			ExecutionService serv =
				theAgent.locate(service.group(), service.system());
			serv.setSessionContext(service.user(), service.id());
			
			instances = serv.queryProcessInstances(filter, sort, threshold);
			if (instances != null) {
				result = new ProcessInstanceWrapper[instances.length];
				for (int i = 0; i < instances.length; i++) {
					result[i] =
						new ProcessInstanceWrapper(
							instances[i].persistentOid(),
							instances[i].name());
				}
			}

		} catch (FmcException ex) {
			System.out.println("Exception occured: " + ex.getMessage());
			return null;
		}
		return result;
	}

	/**
	 * Query work items
	 * @param service session rep to re-create session
	 * @param filter filter string
	 * @param sort sort string
	 * @param threshold threshold
	 * @return array of work item reps
	 */
	public WorkItemWrapper[] queryItems(
		ServiceWrapper service,
		String filter,
		String sort,
		Integer threshold) {
		WorkItem[] items = null;
		WorkItemWrapper[] result = null;
		try {
			// Re-establish session
			ExecutionService serv =
				theAgent.locate(service.group(), service.system());
			serv.setSessionContext(service.user(), service.id());
			
			serv.queryWorkItems(filter, sort, threshold);
			if (items != null) {
				result = new WorkItemWrapper[items.length];
				for (int i = 0; i < items.length; i++) {
					result[i] =
						new WorkItemWrapper(
							items[i].persistentOid(),
							items[i].name());
				}
			}

		} catch (FmcException ex) {
			System.out.println("Exception occured: " + ex.getMessage());
			return null;
		}
		return result;
	}

	/**
	 * Create process instance
	 * @param service session rep to re-create session
	 * @param oidPT process template ID to re-create
	 */
	public void createInstance(ServiceWrapper service, String oidPT) {
		try {			
			// Re-establish session
			ExecutionService serv =
				theAgent.locate(service.group(), service.system());
			serv.setSessionContext(service.user(), service.id());
			
			// Re-establish process template
			ProcessTemplate templ = serv.persistentProcessTemplate(oidPT);

			templ.createInstance("MyInstance", null, null, false);

		} catch (FmcException ex) {
			System.out.println("Exception occured: " + ex.getMessage());
			return;
		}
		return;
	}

	/**
	 * Delete process instance
	 * @param service session rep to re-create session
	 * @param oidPI process instance ID to delete
	 */
	public void deleteInstance(ServiceWrapper service, String oidPI) {
		try {
			// Re-establish session
			ExecutionService serv =
				theAgent.locate(service.group(), service.system());
			serv.setSessionContext(service.user(), service.id());
			
			// Re-establish process instance
			ProcessInstance inst = serv.persistentProcessInstance(oidPI);

			inst.delete();

		} catch (FmcException ex) {
			System.out.println("Exception occured: " + ex.getMessage());
			return;
		}
		return;

	}

}
